'
'ISA Uninstall
'Script for removing ISAPI web filter from an ISA server
'

Option Explicit

'-----------------------------------------------------------------
'                               Host
'-----------------------------------------------------------------

Dim HostName
Dim HostNameDefault

'-----------------------------------------------------------------
'                              Filter
'-----------------------------------------------------------------

Dim FilterName
Dim FilterPath
Dim FilterPathDefault
Dim FilterDLLName
Dim FilterDesc

'-----------------------------------------------------------------
'                                ISA
'-----------------------------------------------------------------

Dim FilterGUID 						'for ISA install
Dim FilterRelativePath				'for ISA install
Dim FilterVendor					'for ISA install
Dim FilterVersion					'for ISA install

'-----------------------------------------------------------------
'                            Parameters
'-----------------------------------------------------------------

HostNameDefault = "LocalHost"
FilterPathDefault = "C:\Program Files\Microsoft ISA Server\AQTRONIX WebKnight\"

FilterName = "WebKnight"
FilterDLLName = "WebKnight.dll"
FilterDesc = "AQTRONIX WebKnight - Web Application Firewall for ISA Server"

FilterGUID = "{6EF039AA-EE06-485a-86E0-E48262241B99}"
FilterRelativePath = "\AQTRONIX WebKnight\" 		'relative path for ISA install!!!
FilterVendor = "AQTRONIX"
FilterVersion = "1.0"

'uncomment the two lines below for unattended setup
HostName = HostNameDefault			    	'uncomment this for unattended setup
'FilterPath = FilterPathDefault				'uncomment this for unattended setup


UninstallISA



'-----------------------------------------------------------------
'                           ISA Install
'-----------------------------------------------------------------

Sub InstallISA()
	'ask for hostname if we don't have one
	If HostName = "" Then
		HostName = AskHostName("Specify the ISA Server where you want to install " & FilterName)
	End If
	If HostName <> "" Then 'user did not cancel?
		If FilterPath = "" Then 'ask for path to install the filter into
			FilterPath = AskLocalPath
		End If
		If FilterPath <> "" Then 'user did not cancel?
			If Left(FilterPath,1)="\" Then	'no UNC paths!
				MsgBox "UNC paths not allowed!",,FilterName
			Else
				'add trailing slash if not there
				If Right(FilterPath,1)<>"\" And Right(FilterPath,1)<>"/" Then
					FilterPath = FilterPath & "\"
				End If

				'ISA root
				Dim FPC
				Set FPC = WScript.CreateObject("FPC.Root")
				'http://msdn.microsoft.com/en-us/library/aa503410.aspx

				'array
				Dim isaArray
				'ISA 2000
				'Set isaArray = FPC.Arrays.GetContainingArray
				'ISA 2000/2004/2006
				Set isaArray = FPC.Arrays(1)

				'list of installed filters
				Dim Filters
				Set Filters = isaArray.Extensions.WebFilters
	
				'copy files to host
				If CopyContent(HostName,FilterPath) Then

					'register filter in ISA
					Dim Filter
					Set Filter = Filters.Add(FilterGUID,FilterName, FilterRelativePath & FilterDLLName , 2, 2)   '2 == fpcFilterPriority_High
					'Filters.MoveDown  1
					Filter.Description = FilterDesc
					Filter.Vendor = FilterVendor
					Filter.Version = FilterVersion
					Filter.Enabled = True
					Filters.Save
					
					'notify user
					MsgBox FilterName & " is succesfully installed on " & HostName & vbCRLF & vbCRLF & "You should review the settings of " & FilterName & " (by running config.exe in " & FilterPath & ") and restart ISA Server on " & HostName & " to load the filter!",,FilterName

					'restart ISA: not necessary for web filter, restartmask = 0!
					'If MsgBox("Do you want to restart ISA server now?",vbYesNo,FilterName)=vbYes Then
					'	Dim restartMask
					'	restartMask = filter.GetServiceRestartMask()
					'	isaArray.RestartServices restartMask
					'End If

					Set Filter = Nothing
				End If
				

				'cleanup
				Set Filters = Nothing
				Set isaArray = Nothing
				Set FPC = Nothing
			End If
		End If
	End If
End Sub

'-----------------------------------------------------------------
'                          ISA Uninstall
'-----------------------------------------------------------------

Sub UninstallISA()
	'ask for hostname if we don't have one
	If HostName = "" Then
		HostName = AskHostName("Specify the ISA Server from which you want to have " & FilterName & " uninstalled")
	End If
	If HostName <> "" Then 'user did not cancel?

		'ISA root
		Dim FPC
		Set FPC = WScript.CreateObject("FPC.Root")
		'http://msdn.microsoft.com/en-us/library/aa503410.aspx

		'array
		Dim isaArray
		'ISA 2000
		'Set isaArray = FPC.Arrays.GetContainingArray
		'ISA 2000/2004/2006
		Set isaArray = FPC.Arrays(1)

		'list of installed filters
		Dim Filters
		Set Filters = isaArray.Extensions.WebFilters
	
		'filter
		Dim Filter
		Set Filter = Filters(FilterGUID) 'this will result in an error if the filter is not found

		'get directory of installed filter
		FilterPath = ".." & Filter.RelativePath
		FilterPath = Left(FilterPath,Len(FilterPath)-Len(FilterDLLName))

		'remove trailing '\' (stupid DeleteFolder does not accept it)
		Do While Right(FilterPath,1)="\"
			FilterPath = Left(FilterPath,Len(FilterPath)-1)
		Loop
		
		'convert dir to unc path or leave it if it's local
		Dim uncpath
		uncpath = FolderToUNC(HostName,FilterPath)
		
		
		Filters.Remove(FilterGUID)
		Filters.Save

		'restart ISA: not necessary for web filter, restartmask = 0!
		'If MsgBox("Do you want to restart ISA server now?",vbYesNo,FilterName)=vbYes Then
		'	Dim restartMask
		'	restartMask = filter.GetServiceRestartMask()
		'	isaArray.RestartServices restartMask
		'End If

		'remove installed dir if yes
		If MsgBox("Do you want to completely remove the directory " & uncPath & " as well?",vbYesNo,FilterName)=vbYes Then
			DeleteFolder uncpath
		End If	

		'notify user
		MsgBox FilterName & " was succesfully uninstalled.",,FilterName

		'cleanup
		Set Filter = Nothing
		Set Filters = Nothing
		Set isaArray = Nothing
		Set FPC = Nothing
	End If
End Sub

'-----------------------------------------------------------------
'                        File System
'-----------------------------------------------------------------

Function CopyContent(HostName,Path)
	'copy files to host
	Dim ret: ret = True
	Dim src
	Dim dest: dest = FolderToUNC(HostName,Path)
	Dim fso
	Set fso = CreateObject("Scripting.FileSystemObject")
	src = fso.GetAbsolutePathName(FilterDLLName)
	src = Left(src,Len(src)-Len(filterDLLName)) & "*.*"

	On Error Resume Next
	fso.CreateFolder dest
	Err.Clear
	fso.CopyFile src, dest, False
	If Err Then
		If MsgBox("Error copying the files to " & dest & ". If you know the error (access denied, default shares don't exist,...). You can continue setup and copy all the files manually. Click Yes to continue setup, No to cancel the installation." & vbCRLF & vbCRLF & "The specified error was: " & err.number & ": " & err.Description ,vbYesNo,FilterName)=vbNo Then
			ret = False
		End If
	End If
	On Error Goto 0

	CopyContent = ret
End Function

Sub DeleteFolder(path)
	Dim fso
	Set fso = CreateObject("Scripting.FileSystemObject")
	On Error Resume Next
	fso.DeleteFolder path, True
	if Err Then
		MsgBox "There was an error deleting the files. You should do it manually, setup will continue." & vbCRLF & vbCRLF & "The specified error is: " & err.Number & ": " & err.Description,,FilterName
	End If
	On Error Goto 0
End Sub

Function FolderToUNC(HostName,Path)
	'convert dir to unc path or leave it if it's local
	If LCase(HostName)="localhost" Then
		FolderToUNC = Path
	Else
		FolderToUNC = "\\" & HostName & "\" & Replace(Path,":","$")
	End If
End Function

Sub SetNTFSPermission(Path,Username,Permission)
	'http://www.experts-exchange.com/Programming/Installation/InstallShield/Q_24315443.html
	If Right(Path,1) = "\" Then Path = Left(Path,Len(Path)-1)
	'MsgBox "cacls.exe """ & Path & """ /T /E /C /G """ & Username & """:" & Permission,,"Command"
	CreateObject("WScript.Shell").run "cacls.exe """ & Path & """ /T /E /C /G """ & Username & """:" & Permission
End Sub

'-----------------------------------------------------------------
'                             Prompts
'-----------------------------------------------------------------

Function AskHostName(Message)
    'HostNameDefault has to be declared
    AskHostName = InputBox(Message,"Hostname required",HostNameDefault)
End Function

Function AskLocalPath()
    'HostName, FilterName, FilterPathDefault has to be declared
    AskLocalPath = InputBox("Specify a path on " & HostName & " where you want " & FilterName & " to be installed. This has to be an absolute path (with a local drive letter on that host, no UNC allowed here!)","Path required", FilterPathDefault)
End Function