/*
    AQTRONIX C++ Library
    Copyright 2011 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// SyslogDevice.h: interface for the CSyslogDevice class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SyslogDevice.h"

//2011.01.07 class created

/*
 * SYSLOG RFC 3164
 * http://tools.ietf.org/html/rfc3164
 *
 * SYSLOG = <PRI> TIMESTAMP HOSTNAME MSG
 * PRI = number
 * TIMESTAMP = "Mmm dd hh:mm:ss" (without quotes)
 * HOSTNAME = string without domain or IPv4 or IPv6
 * MSG = string or TAG + CONTENT
 * TAG = application name: or application name[pid]:
 * CONTENT = string
 */

#define CSYSLOGDEVICE_TIMESTAMP "%b %d %H:%M:%S"

CSyslogDevice::CSyslogDevice():m_IsReady(false),DestinationPort(514),DestinationHost("localhost"),Enabled(false),LastError(""),SourceHostname(""),ApplicationName("")
{

}

CSyslogDevice::~CSyslogDevice()
{

}

void CSyslogDevice::Initialize()
{
	try{
		if(AfxSocketInit()==TRUE){
			SourceHostname = DetectHostname();
			BOOL ret = Create(0,SOCK_DGRAM,FD_WRITE);
			if(ret!=TRUE){
				ReportError(GetLastError());
			}else{
				m_IsReady = true;
			}
		}else{
			ReportError(GetLastError());
		}
	}catch(...){
		ReportError(GetLastError());
	}
}

CString CSyslogDevice::DetectHostname()
{
	try{
		TCHAR chrComputerName[MAX_COMPUTERNAME_LENGTH + 1];
		DWORD dwBufferSize = MAX_COMPUTERNAME_LENGTH + 1; 
		if(GetComputerName(chrComputerName,&dwBufferSize)) {
			return chrComputerName;
		} else {
			return "";
		} 
	}catch(...){
		return "";
	}
}

void CSyslogDevice::OnSend(int nErrorCode)
{
	m_IsReady = true;
	CAsyncSocket::OnSend(nErrorCode);
}

bool CSyslogDevice::Send(CTime& Timestamp, CString& Msg)
{
	if(!m_IsReady)
		return false;

	//pri + timestamp
	CString LogMsg = "<";
	LogMsg += Priority;
	LogMsg += ">" + Timestamp.Format(CSYSLOGDEVICE_TIMESTAMP) + " ";
	//source hostname
	if(SourceHostname!=""){
		LogMsg += SourceHostname + " ";
	}
	//application name
	if(ApplicationName!=""){
		LogMsg += ApplicationName + ": ";
	}
	//message
	LogMsg += Msg;
	//send
	return Send(LogMsg.GetBuffer(),LogMsg.GetLength()+1);
}

bool CSyslogDevice::Send(const void* pBuf, int nBufLen)
{
	if(!m_IsReady)
		return false;

	try{
		int ret = CAsyncSocket::SendToEx((LPCTSTR)pBuf,nBufLen,DestinationPort,DestinationHost);
		if(ret==SOCKET_ERROR){
			UINT uErr = GetLastError();
			if(uErr==WSAEWOULDBLOCK){
				m_IsReady = false;
			}else{
				ReportError(uErr);
			}
			return false;
		}
		return true;
	}catch(...){
		LastError = "Fatal error sending syslog message";
		return false;
	}
}

void CSyslogDevice::ReportError(UINT uErr)
{
	TCHAR szError[256];
	wsprintf(szError, "Socket Error: %d", uErr);
	LastError = szError;
	//AfxMessageBox(szError);
}