/*
	AQTRONIX WebKnight - ISAPI Filter for securing IIS
	Copyright 2002-2008 Parcifal Aertssen

	This file is part of AQTRONIX WebKnight.

    AQTRONIX WebKnight is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

    AQTRONIX WebKnight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX WebKnight; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
// WEBKNIGHT.CPP - Implementation file for your Internet Server
//    WebKnight Filter


#include "stdafx.h"
#include "WebKnight.h"

#define STRING2(x) #x
#define STRING(x) STRING2(x)

#ifdef _MSC_VER
#pragma message("***** _MSC_VER:" STRING(_MSC_VER) "*****")
#if _MSC_VER > 1200
						//VS 6.0 = MSC 1200
						//VS 2005 = MSC 1400
#else
#pragma message("***** To avoid errors, make sure you add the project Library folder to your include sources (Tools->Options->Directories). *****")
#endif
#endif

/* AQTRONIX WebKnight
 *
 * See README.TXT for information about the source code
 *
 * 13 August    2002: Project started
 * 26 October   2002: Started coding a GUI for settings
 *  4 November  2002: Beta 1.0 released (without GUI)
 * 12 January   2003: First version of GUI finished
 * 25 February  2003: Windows Installer package finished
 * 24 March     2003: Final 1.0 released (+config 1.0)
 * 14 July      2003: Final 1.1 released (+config 1.1)
 *  8 September 2003: Final 1.2 released (+config 1.1)
 * 10 November  2003: Final 1.3 released (+config 1.2)
 * 24 December  2006: Final 2.0 released (+config 1.3 + loganalysis 1.0)
 *  8 October   2007: Final 2.1 released (+config 1.3 + loganalysis 1.1)
 *  2 September 2008: Final 2.2 released (+config 1.4 + loganalysis 1.2)
 *  3 April     2010: Final 2.3 released (+config 1.5 + loganalysis 1.3)
 * 29 December  2010: Final 2.4 released (+config 1.5 + loganalysis 1.4)
 *
 * You can not define _UNICODE because this program is not made for it and the
 * ISAPI framework does not (yet) support Unicode (I use Visual C++ 6.0).
 * If you still want to enable this you have to review every piece of code and
 * adjust all char types, "str" functions,...
 *
 * For IIS6 (and up) you cannot register for the OnReadRawData event if IIS is
 * running in Worker Process mode (only works in IIS 5.0 Isolation mode).
 *
 * Making default settings (pre WebKnight 2.0 Final):
 * - remove logdir
 * - remove denied ip addresses
 * - uncheck UseDenyIPList
 * - remove denied user agents
 * - uncheck UseDenyHeaderUserAgent
 * - remove denied user agent sequences
 * - uncheck UseDenyHeaderUserAgentSequences
 */

/* TODO
 * -----
 * WEBKNIGHT
 *  <2.5>
 * - WebKnight ISA Server installation
 *		+ LoadISACompatibility
 *		+ do not register for SF_NOTIFY_URL_MAP
 *		+ WebKnight settings/aparte download?
 * - Exclude certain urls from being scanned like /admin or /public like URLScan's "alwaysallowedurls"
 *
 *	<3.0>
 * - IPv6 problem on W2008
		MonitorIPRangeList.Enumerate(Settings.Connection.MonitorAddresses.List);
		DenyIPRangeList.Enumerate(Settings.Connection.DenyAddresses.List);
		ExcludedIPRangeList.Enumerate(Settings.Engine.ExcludedIPAddresses.List);
		127.0.0.1 -> ::1
 * - IIS 6 isapi wildcard alternative to global filtering
 * - fully multithreaded scanning engine (not just thread safe)
 * - log file per web instance / single file logging like urlscan 3.0 beta
 * - per process owner settings
 * - application/x-www-UTF8-encoded & UTF (url,query,header,data) + check invalid UTF sequences
 *
 * FEATURES UNDER CONSIDERATION
 * - fake user agents:  - cloak robots.txt Disallow: /
 *						- first request is for robots.txt -> block it
 * - captcha alternative / data analysis to detect bots and spamming POST messages
 * 		- http://www.stopforumspam.com/apis (90.152.30.61 zit er in en heeft gespamd)
 *		- http://isc.sans.org/diary.html?storyid=1836
 *		- session not initialized or no jpg/gif/css file requested
 *		- http://www.usenix.org/events/nsdi05/tech/kandula/kandula_html/
 * - Robots database auto update
 * - log to syslog
 *
 * WANTED FEATURES
 * - Central Policy Management: Settings in local file or Active Directory (+schedules, +organizational unit, +multiple policies)
 * - Real-Time Statistics
 * - Logging to ODBC Database
 * - MMC or tabsheet plugin for IIS: using own GUI XML protocol for future compatibility
 *
 * REJECTED FEATURES
 * - scan user agent anomalies (unusual http header sequence,...)
 * - print loaded settings (only options) to the log file (when a load/reload of settings happens) REJECTED: too much settings, this is not urlscan!
 * - compare settings with previously loaded settings when printing settings to log file
 * - validHostport (see kodeit), this is stupid because of HTTP 0.9 and HTTP 1.0 requests still possible
 * - Logging: W3C extended log format: too limited!
 * - Logging backup: database logging or backup can be scheduled by OS
 * - Settings in Registry: why use that when you have XML or LDAP?
 *
 * POSSIBLE EXPLOITS IN IIS:
 * - %00 exploit?
 * - Ms-Echo-Request header??
 * - VERBS (from winhttp.dll & SEARCH exploit?)
 * - headers (from winhttp.dll)
 * - encoding exploits UTF-8, Shift-JIS, EUC-JP...
 *
 * BUGS IN WEBKNIGHT
 * - Settings: INI file: writing ';' in SQL Keywords cannot be read by ReadFrom function -> WILL NOT BE FIXED (Reason: INI settings not used by default)
 *
 * BUGS IN ISAPI FRAMEWORK:
 * 1) if attack detected in OnReadRawData -> no OnSendRawData event (server header not
 *    changed or removed and filtercontext not reset). SOLUTION: added code in HackResponse().
 *    Only works with ResponseDirectly=1! If you redirect to a url or only send back a
 *    ResponseStatus then server header is not changed.
 */

///////////////////////////////////////////////////////////////////////
// The one and only CWebKnightFilter object

CWebKnightFilter theFilter;

///////////////////////////////////////////////////////////////////////
// The one and only CWebKnightExtension object

CWebKnightExtension theExtension;

