/*
    AQTRONIX C++ Library
    Copyright 2009-2010 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
#include "StdAfx.h"
#include "WebKnightFilter.h"

///////////////////////////////////////////////////////////////////////
// CWebKnightFilter implementation

CWebKnightFilter::CWebKnightFilter():CISAPIFirewall(Settings){

	CString strinit; CString strinit2; CString strinit3;

	//get the directory of the .dll file & windows directory
	CFileName dll = GetProcessPath("WebKnight.dll");
	CString windir = GetWindowsPath();

	CSingleLock lock(&crit, TRUE);

	strinit = LoadSettingFile(dll.Path,"WebKnight.xml","WebKnight.ini","urlscan.ini");
	strinit2 = SaveDefaultSettingFile(dll.Path,"WebKnight.xml");

	CString logdir;
#ifdef SYSTEM_LOG_BUILD
	logdir = windir + "\\System32\\LogFiles\\WebKnight\\"; //for global filter
	#pragma message("***** This build will log under the LogFiles folder of your Windows NT installation! *****")
#else
	logdir = dll.Path  + "LogFiles\\";				   //<dir of dll>\LogFiles
#endif
	strinit3 = InitializeLogger(logdir,APP_FULLNAME);

	logger.NewEntryFlush("AQTRONIX WebKnight loaded");
	logger.NewEntryFlush(strinit);

	if(strinit2!="")
		logger.NewEntryFlush(strinit2);
	if(strinit3!="")
		logger.NewEntryFlush(strinit3);

	logger.ErrorEntry("If you are running multiple instances of WebKnight then only the first loaded instance has access to the log file. Make sure each instance of WebKnight has a unique log file, you can also enable Per Process Logging or Per Process Owner Logging.");

	//are we in logging only mode? if so, warning!
	WarnLogonlyMode();

	Settings.ReadAgentsFile(dll.Path + "Robots.xml");

	LoadCache();

	SaveRunningValues(dll.Path,"Loaded.xml");

	lock.Unlock();
}

CWebKnightFilter::~CWebKnightFilter()
{
	CSingleLock lock(&crit, TRUE);
	logger.NewEntryFlush("AQTRONIX WebKnight unloaded");
	lock.Unlock();
}

BOOL CWebKnightFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);
	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	CSingleLock lock(&crit, TRUE);

	SetFilterVersion(pVer);

	pVer->dwFlags	|= SF_NOTIFY_SEND_RAW_DATA | SF_NOTIFY_URL_MAP
					| SF_NOTIFY_PREPROC_HEADERS | SF_NOTIFY_AUTHENTICATION;

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);

	lock.Unlock();
	return TRUE;
}

DWORD CWebKnightFilter::OnReadRawData(CHttpFilterContext* pCtxt,PHTTP_FILTER_RAW_DATA pRawData)
{
	return CISAPIFirewall::OnReadRawData(pCtxt,pRawData,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnPreprocHeaders(CHttpFilterContext* pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders) 
{
	return CISAPIFirewall::OnPreprocHeaders(pfc,pHeaders,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnUrlMap(CHttpFilterContext* pfc, PHTTP_FILTER_URL_MAP pUrlMap) 
{
	return CISAPIFirewall::OnUrlMap(pfc,pUrlMap,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnAuthentication(CHttpFilterContext* pfc, PHTTP_FILTER_AUTHENT pAuthent) 
{
	return CISAPIFirewall::OnAuthentication(pfc,pAuthent,SITEINSTANCE,APP_SERVER_HEADER);
}

DWORD CWebKnightFilter::OnSendRawData(CHttpFilterContext* pfc, PHTTP_FILTER_RAW_DATA pRawData)
{
	return CISAPIFirewall::OnSendRawData(pfc,pRawData,SITEINSTANCE);
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CWebKnightFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CWebKnightFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/


void CWebKnightFilter::LoadCache()
{
	CHTTPFirewall::LoadCache();

	BOT_Blocked.SetMaxAge(CTimeSpan(0,Settings.Robots.BotTimeout>1?Settings.Robots.BotTimeout:1,0,0));
	BOT_Aggressive.SetMaxAge(CTimeSpan(0,0,Settings.Robots.DenyAggressive.MaxTime>1?Settings.Robots.DenyAggressive.MaxTime:1,0));
}

bool CWebKnightFilter::CustomHeaderScanning(CHttpFilterContext *pfc, PHTTP_FILTER_PREPROC_HEADERS pHeaders, CString& ip, CURL& RawUrl)
{
	bool hack(false);
	CFileName fn(CURL::URLEncodedToASCII(RawUrl.Url));

	//BOTS
	CString ua(GetHeader(pfc,pHeaders,"User-Agent:"));
	logger.Append(ua.Left(2048));

	bool bRobotFile(false);

	//BOTS
	if(fn.FileNameLCase=="robots.txt"){
		//BOTS - Robots.txt
		if(Settings.Robots.AllowRobotsFile){
			logger.Append("File access 'robots.txt'");
			bRobotFile = true;
		}
		//BOTS - Block All
		if(Settings.Robots.DenyAll){
			BOT_Blocked.Add(ip,ua);
			logger.Append("BLOCKED: Robots not allowed");
		}
		//BOTS - Deny aggressive - start monitoring
		if(Settings.Robots.DenyAggressive.Enabled){
			//add request to cache to start monitoring
			BOT_Aggressive.Add(ip,ua);
		}
	}else{
		//BOTS - Block Aggressive Bots - monitor subsequent requests after robots.txt
		if(Settings.Robots.DenyAggressive.Enabled){
			if(BOT_Aggressive.Exists(ip,ua)){
				//add request to cache
				BOT_Aggressive.Add(ip,ua);

				//block if number of request exceeds certain treshold
				if(BOT_Aggressive.Count(ip)>Settings.Robots.DenyAggressive.MaxCount){
					BOT_Blocked.Add(ip,ua);
					logger.Append("BLOCKED: Aggressive robot not allowed");
				}
			}
		}
	}

	//BOTS - Block Bad Bots
	if(Settings.Robots.DenyBad){
		if( DenySequences(RawUrl.Url,Settings.Robots.BadBotTraps,"BLOCKED: Bad robot fell for trap: '","'") ){
			BOT_Blocked.Add(ip,ua);
		}
	}

	if(!bRobotFile){
		//BOTS - Block if in list
		if(BOT_Blocked.Exists(ip,ua)){
			hack = true;
			logger.Append("BLOCKED: Robot not allowed until timeout expires");
		}

		//HEADERS: User Agent
		hack |= ScanUserAgent(ua);
	}

	return hack;
}

bool CWebKnightFilter::CustomSendRawData(CHttpFilterContext *pfc, PHTTP_FILTER_RAW_DATA pRawData, CString& Response)
{
	/*
	 * Change server header or remove it.
	 * Log only if an error occured, but deny nothing!
	 */ 
	bool doflush(false);
	//if we have to do something with server header
	if (Settings.Headers.RemoveServerHeader||Settings.Headers.ChangeServerHeader){

		if(ChangeRawDataHeader(Response,"Server:",Settings.Headers.ServerHeader,Settings.Headers.RemoveServerHeader)){
			char* p = (char*) pfc->AllocMem(Response.GetLength()+1);
			if(p!=NULL){
				strcpy(p,(LPCTSTR)Response);
				pRawData->pvInData = p;
				pRawData->cbInData = Response.GetLength();
				pRawData->cbInBuffer = Response.GetLength()+1;
				logger.Append("INFO: Changed/removed 'Server:' header");
			}else{
				logger.Append("WARNING: Error changing/removing 'Server:' header (could not allocate memory!)");
				doflush = true;
			}
		}else{
			logger.Append(Response); //Response contains error message if false returned
			doflush = true;
		}
	}
	return doflush;
}
