'
'install.vbs
'Script for adding WebKnight in the IIS metabase on localhost using current directory
'
Option Explicit

Dim HostName
Dim FilterName
Dim FilterDLLName
Dim FilterPath
Dim FilterDesc

FilterName = "WebKnight"
FilterDLLName = "WebKnight.dll"
FilterDesc = "Filter for securing IIS"

'uncomment the two lines below for unattended setup
HostName = "LocalHost"
FilterPath = Session.Property("WEBKNIGHTDIR")
If FilterPath="" Then
	FilterPath = Session.Property("CustomActionData")
End If

RegisterIISFilter FilterName,HostName,FilterPath

'-----------------------------------------------------------------
'                           IIS Install
'-----------------------------------------------------------------

Sub RegisterIISFilter(FilterName,HostName,FilterPath)
	'Install as IIS Filter
	Dim IISVersion
	Dim UseNewInstaller: UseNewInstaller = False

	'Get IIS Version
	IISVersion = GetIISVersion()
	If IISVersion <> "" Then
	    If IsNumeric(IISVersion) Then
	        If IISVersion > 6 Then
	            UseNewInstaller = True
	        End If
	    End If
	End If

	'Install
	If UseNewInstaller Then
	    RegisterIIS7Filter FilterName,FilterPath
	Else
	    RegisterIISFilterMetabase FilterName,HostName,FilterPath
	End If
End Sub

Sub RegisterIISFilterMetabase(FilterName,HostName,FilterPath)
    'Install as IIS 5 & 6 Filter
    Dim Filters
    Dim Filter
    Dim LoadOrder

    If Left(FilterPath,1)="\" Then	'no UNC paths!
	    MsgBox "UNC paths not allowed!",,FilterName
    Else
	    'add trailing slash if not there
	    If Right(FilterPath,1)<>"\" And Right(FilterPath,1)<>"/" Then
		    FilterPath = FilterPath & "\"
	    End If

	    'get list of filters on the host
	    Set Filters = GetObject("IIS://" & HostName & "/W3SVC/Filters")
	    LoadOrder = Filters.FilterLoadOrder

	    If InStr(1,LoadOrder,FilterName)>0 Then	'filter already installed?
		    MsgBox FilterName & " is already installed on " & HostName,,FilterName
	    Else
		    'register filter in IIS metabase
		    'set filter to be loaded first (high security)
		    If LoadOrder <> "" Then
			    LoadOrder = "," & LoadOrder
		    End If
		    LoadOrder = FilterName & LoadOrder
		    If InStr(1,LoadOrder,"Compression")>0 Then
			    LoadOrder = Replace(LoadOrder,"Compression","")
			    LoadOrder = Replace(LoadOrder,",,",",")
			    LoadOrder = "Compression," & LoadOrder
		    End If
		    If InStr(1,LoadOrder,"sspifilt")>0 Then
			    LoadOrder = Replace(LoadOrder,"sspifilt","")
			    LoadOrder = Replace(LoadOrder,",,",",")
			    LoadOrder = "sspifilt," & LoadOrder
		    End If
		    Filters.FilterLoadOrder = LoadOrder
		    Filters.SetInfo

		    'make filter
		    Set Filter = Filters.Create("IIsFilter", FilterName)
		    Filter.FilterPath = FilterPath & FilterDLLName
		    Filter.FilterDescription = FilterDesc
		    Filter.SetInfo

		    'notify user
		    MsgBox FilterName & " is succesfully installed." & vbCRLF & vbCRLF & "You should review the settings of " & FilterName & " (by running config.exe in " & FilterPath & ") and restart IIS on " & HostName & " to load the firewall!",,FilterName
	    End If
    End If
End Sub

Sub RegisterIIS7Filter(FilterName,FilterPath)
    'install ISAPI filter using IIS7 API

    Dim AdminManager
    Dim IsapiFiltersSection
    Dim IsapiFiltersCollection
    Dim FilterElement

    'add trailing slash if not there
    If Right(FilterPath,1)<>"\" And Right(FilterPath,1)<>"/" Then
	    FilterPath = FilterPath & "\"
    End If

    Set AdminManager = CreateObject("Microsoft.ApplicationHost.WritableAdminManager")
    AdminManager.CommitPath = "MACHINE/WEBROOT/APPHOST"

    Set IsapiFiltersSection = AdminManager.GetAdminSection("system.webServer/isapiFilters", "MACHINE/WEBROOT/APPHOST")

    Set IsapiFiltersCollection = IsapiFiltersSection.Collection

    Set FilterElement = IsapiFiltersCollection.CreateNewElement("filter")
    FilterElement.Properties.Item("name").Value = FilterName
    FilterElement.Properties.Item("path").Value = FilterPath & FilterDLLName
    'FilterElement.Properties.Item("preCondition").Value = "bitness32"

    IsapiFiltersCollection.AddElement(FilterElement)
     
    AdminManager.CommitChanges()
    
    Set FilterElement = Nothing
    Set IsapiFiltersCollection = Nothing
    Set IsapiFiltersSection = Nothing
    Set AdminManager = Nothing

    'notify user
    MsgBox FilterName & " is succesfully installed." & vbCRLF & vbCRLF & "You should review the settings of " & FilterName & " (by running config.exe in " & FilterPath & ") and restart IIS on " & HostName & " to load the firewall!",,FilterName

End Sub

'-----------------------------------------------------------------
'                         IIS  Version
'-----------------------------------------------------------------

Function GetIISVersion
    'Get IIS Version
    Dim oShell
    Dim v:v = ""
    
    On Error Resume Next
    Set oShell = CreateObject("WScript.Shell")
    v = oShell.RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\InetStp\MajorVersion")
    On Error Goto 0
    Set oShell = Nothing

    GetIISVersion = v
End Function
